@@ -38,6 +38,10 @@ file_exists <- function(path) {
 #' @export
 dir_exists <- function(path) {
   res <- is_dir(path)
+
+  links <- is_link(path)
+  res[links] <- is_dir(link_path(path[links]))
+
   !is.na(res) & res
 }
 
