@@ -75,8 +75,26 @@ path_real <- function(path) {
   path <- enc2utf8(path)
   old <- path_expand(path)
 
+  # We need to convert all paths to absolute paths, but _not_ to normalize
+  # them, so we cannot use `path_abs()`.
+  is_abs <- is_absolute_path(path)
+  old[!is_abs] <- path(getwd(), path[!is_abs])
+
   is_missing <- is.na(path)
-  old[!is_missing] <- realize_(old[!is_missing])
+  exists <- file_exists(path) == TRUE
+
+  # Realize all paths which fully exist
+  old[!is_missing & exists] <- realize_(old[!is_missing & exists])
+
+  # Handle paths which only partially exist
+  realize_one <- function(splits) {
+    paths <- Reduce(fs::path, splits, accumulate = TRUE)
+    last_link <- which.max(is_link(paths))
+    path(realize_(paths[last_link]), path_join(splits[seq(last_link + 1, length(splits))]))
+  }
+
+  partial <- !is_missing & !exists
+  old[partial] <- vapply(path_split(old[partial]), realize_one, character(1))
 
   path_tidy(old)
 }
@@ -339,7 +357,7 @@ path_ext <- function(path) {
     return(character())
   }
 
-  res <- captures(path, regexpr("(?<!^|[.]|/)[.]([^.]+)$", path, perl = TRUE))[[1]]
+  res <- captures(path_file(path), regexpr("(?<!^|[.]|/)[.]([^.]+)$", path_file(path), perl = TRUE))[[1]]
   res[!is.na(path) & is.na(res)] <- ""
   res
 }
@@ -347,13 +365,30 @@ path_ext <- function(path) {
 #' @rdname path_file
 #' @export
 path_ext_remove <- function(path) {
-  path_tidy(sub("(?<!^|[.]|/)[.][^.]+$", "", path, perl = TRUE))
+  dir <- path_dir(path)
+  file <- sub("(?<!^|[.]|/)[.][^.]+$", "", path_file(path), perl = TRUE)
+
+  na <- is.na(path)
+  no_dir <- dir == "." | dir == ""
+
+  path[!na & no_dir] <- path_tidy(file[!na & no_dir])
+
+  path[!na & !no_dir] <- path(dir[!na & !no_dir], file[!na & !no_dir])
+
+  path
 }
 
 #' @rdname path_file
 #' @export
 path_ext_set <- function(path, ext) {
-  path[!is.na(path)] <- paste0(path_ext_remove(path[!is.na(path)]), ".", ext)
+  # Remove a leading . if present
+  ext <- sub("[.]", "", ext)
+
+  has_ext <- nzchar(ext)
+  to_set <- !is.na(path) & has_ext
+
+  path[to_set] <- paste0(path_ext_remove(path[to_set]), ".", ext[to_set])
+
   path_tidy(path)
 }
 
