@@ -11,7 +11,7 @@
 using namespace Rcpp;
 
 // [[Rcpp::export]]
-void mkdir_(CharacterVector path, mode_t mode) {
+void mkdir_(CharacterVector path, unsigned short mode) {
   R_xlen_t n = Rf_xlength(path);
   for (R_xlen_t i = 0; i < n; ++i) {
     uv_fs_t req;
@@ -57,7 +57,6 @@ void dir_map(
   uv_fs_scandir(uv_default_loop(), &req, path, 0, NULL);
 
   if (!fail && warn_for_error(req, "Failed to search directory '%s'", path)) {
-    uv_fs_req_cleanup(&req);
     return;
   }
 
